<?php

// If this file is called directly, abort.
if (!defined('ABSPATH')) exit;

/**
 * Class Eurofaktura_order_meta_box
 *
 * Handles the display and actions of the custom meta box on WooCommerce orders.
 *
 * @since 1.0.0
 */
class Eurofaktura_order_meta_box
{
    /**
     * Holds data related to Eurofaktura.
     *
     * @var array|null
     */
    private $er_data = null;

    /**
     * Get Eurofaktura data.
     *
     * @return array|null
     */
    public function get_er_data()
    {
        return $this->er_data;
    }

    /**
     * Set Eurofaktura data.
     *
     * @param array $er_data Eurofaktura data.
     */
    public function set_er_data($er_data)
    {
        $this->er_data = $er_data;
    }

    /**
     * Constructor.
     * Adds actions to WordPress hooks.
     */
    public function __construct()
    {
        add_action("add_meta_boxes", [$this, "add"]);
        add_action("wp_ajax_eurofaktura_order_stock", [$this, "order_stock"]);
        add_action("wp_ajax_eurofaktura_document_pdf", [$this, "document_pdf"]);
        add_action("wp_ajax_eurofaktura_order_synchronize", [$this, "order_synchronize"]);
        add_action("wp_ajax_eurofaktura_parcel_refresh", [$this, "parcel_refresh"]);
        add_action("wp_ajax_eurofaktura_order_synchronize", [$this, "order_synchronize"]);
        add_action("wp_ajax_eurofaktura_parcel_pdf", [$this, "parcel_pdf"]);
    }

    /**
     * Adds the custom meta box to WooCommerce order screen.
     */
    public function add()
    {
        add_meta_box(
            "eurofaktura-order-meta-box",
            eurofaktura_get_title(),
            [$this, "render"],
            wc_get_page_screen_id("shop-order"),
            "side",
            "high"
        );
    }

    /**
     * Renders the content of the custom meta box.
     *
     * @param WP_Post|WC_Order $order The order object.
     */
    public function render($order)
    {

        if ($order instanceof WC_Order) {
        } elseif ($order instanceof WP_Post && 'shop_order' === $order->post_type) {
            $order = wc_get_order($order);
        } else {
            exit;
        }

        $order_id = $order->get_id();
        $can_do_actions = false;


        if (get_option("eurofaktura_connected")) {
            $client = Eurofaktura_http_client::instance();
            $response = $client->post_request('order.refresh', ['order_number' => strval($order_id)]);

            $webhook = get_option('eurofaktura_webhook_id');
            $webhook = eurofaktura_get_active_webhook($webhook);
            $notice_message = sprintf(
                // translators: %1$s -> Selected webhook url. 
                __('No response from server on url: %1$s. Please check if selected Webhook Url is correct.', 'eurofaktura'),
                eurofaktura_get_webhook_delivery_url($webhook)
            );

            if (isset($response) && !empty($response)) {
                $response_status_code = $response->getStatusCode();
                $response_body = $response->getBody()->getContents();
                $response_body = json_decode($response_body, true);

                if (isset($response_body['message'])) {
                    $notice_message = $response_body['message'];
                }

                if ($response_status_code === 200) {
                    $response_body_data = $response_body['data'];
                    $this->set_er_data($response_body_data['metaData']);
                    $can_do_actions = true;
                }
            }
        }
?>
        <div class="eurofaktura-meta-box">
            <div class="eurofaktura-meta-box-main">
                <?php

                if ($can_do_actions == true) {
                    foreach ($this->get_er_data() as $data) {

                        $documentID = isset($data['documentID']) ? $data['documentID'] : '';
                        $actions_html = '';
                        $message_html = '';
                        if (isset($data['actionsLine1Message'])) {
                            $message_html = '<p id="' . $documentID . '">' . $data['actionsLine1Message'] . '</p>';
                        }
                        foreach ($data['actionsLine1'] as $action) {

                            $action_html = '';
                            if ($action['type'] == 'link') {
                                $href = $action['action'];
                                $name = $action['title'];
                                $action_html = '<a href="' . $href . '" target="eurofaktura" class="eurofaktura-button-link">' . $name . '</a>';
                            } elseif ($action['type'] == 'action') {
                                $ajaxAction = $action['action'];
                                $name = $action['title'];
                                $action_html = '<button type="button" class="eurofaktura-button-link eurofaktura-' . $ajaxAction . '" data-eurofaktura-document-id="' . $documentID . '">' . $name . '</button>';
                            }

                            if (empty($actions_html)) {
                                $actions_html =  $action_html;
                            } else {
                                $actions_html = $actions_html . '<span> | </span>' . $action_html;
                            }
                        }

                ?>
                        <div class="eurofaktura-meta-box-section">
                            <h3 class="eurofaktura-meta-box-section-header">
                                <span>
                                    <img class="eurofaktura-meta-box-icon" src="<?php echo esc_url(eurofaktura_get_icon_src_from_symbol($data['icon'])); ?>" />
                                </span>
                                <?php echo esc_html($data['title']); ?>
                            </h3>
                            <?php echo wp_kses_post($message_html); ?>
                            <?php echo wp_kses_post($actions_html); ?>
                        </div>
                    <?php

                    }
                } else {

                    if (get_option("eurofaktura_connected")) {

                    ?>
                        <label><?php echo esc_html($notice_message) ?></label>
                    <?php

                    } else {
                    ?>
                        <label><?php echo esc_html(sprintf(
                                    // translators: 1.Eurofaktura product title. 
                                    __('Connection to %1$s is not established.', 'eurofaktura'),
                                    eurofaktura_get_title()
                                )); ?></label>
                <?php
                    }
                } ?>
            </div>

            <div class="eurofaktura-meta-box-footer">
                <?php if (eurofaktura_is_admin()) { ?>
                    <img class="eurofaktura-meta-box-icon" src="<?php echo esc_url(eurofaktura_get_icon_src_from_symbol('gearSettings')); ?>" />
                    <a class="eurofaktura-button-link" href="<?php echo esc_url(admin_url("admin.php?page=wc-settings&tab=eurofaktura")); ?>">
                        <?php echo esc_html(__('Settings', 'eurofaktura')); ?></a>
                <?php } ?>
            </div>
        </div>
<?php

    }

    /**
     * Handles AJAX request for order synchronization.
     */
    public function order_synchronize()
    {
        if (!wp_verify_nonce($_POST['_wpnonce'])) {
            wp_die('Coud not verify nonce');
        }
        $wc_order_id = sanitize_text_field($_POST['wc_order_id']);

        $order = wc_get_order($wc_order_id);
        $order_array = eurofaktura_map_order_for_request($order);

        $client = Eurofaktura_http_client::instance();
        $response = $client->post_request('order.synchronize', $order_array);

        $response_message = 'Unexpected error';
        $response_custom_status = '';

        if (!empty($response)) {
            $response_status_code = $response->getStatusCode();
            $response_body = $response->getBody()->getContents();
            $response_body = json_decode($response_body, true);

            if (isset($response_body['message'])) {
                $response_message = $response_body['message'];
            }

            if (isset($response_body['status'])) {
                $response_custom_status = $response_body['status'];
            }

            if ($response_status_code === 200) {
                if ($response_custom_status == 'orderSynchronizeWarning') {
                    Eurofaktura_admin_notices::instance()->add_admin_notice($response_message, 'eurofaktura_order', 'warning');
                    exit;
                } else {
                    Eurofaktura_admin_notices::instance()->add_admin_notice($response_message, 'eurofaktura_order', 'success');
                    exit;
                }
            }
        }
        Eurofaktura_admin_notices::instance()->add_admin_notice($response_message, 'eurofaktura_order', 'error');
        exit;
    }

    /**
     * Handles AJAX request for generating PDF document.
     */
    public function document_pdf()
    {
        if (!wp_verify_nonce($_POST['_wpnonce'])) {
            wp_send_json_error(array(
                'message' => 'Could not verify nonce'
            ));
            exit;
        }
        $document_id = sanitize_text_field($_POST['document_id']);

        $client = Eurofaktura_http_client::instance();
        $response = $client->post_request('document.pdf', ['document_id' => strval($document_id)]);

        if (!empty($response)) {
            $response_status_code = $response->getStatusCode();

            if ($response_status_code === 200) {
                $response_body = $response->getBody()->getContents();
                $content_disposition = $response->getHeader('Content-Disposition');

                $filename = 'document.pdf';

                if (!empty($content_disposition)) {
                    preg_match('/filename="([^"]+)"/', $content_disposition[0], $matches);
                    if (count($matches) > 1) {
                        $filename = $matches[1];
                    }
                    wp_send_json_success(array(
                        'filename' => $filename,
                        'contents' => base64_encode($response_body)
                    ));
                    wp_die();
                }
            }
        }
        wp_send_json_error();
        wp_die();
    }

    /**
     * Handles AJAX request for refreshing parcel information.
     */
    public function parcel_refresh()
    {
        if (!wp_verify_nonce($_POST['_wpnonce'])) {
            wp_send_json_error(array(
                'message' => 'Could not verify nonce'
            ));
            exit;
        }
        $document_id = sanitize_text_field($_POST['document_id']);

        $client = Eurofaktura_http_client::instance();
        $response = $client->post_request('parcel.refresh', ['document_id' => strval($document_id)]);

        if (!empty($response)) {

            $response_body = $response->getBody()->getContents();
            $response_body = json_decode($response_body, true);

            wp_send_json_success(array(
                'message' => $response_body['message']
            ));
            wp_die();
        }
        wp_send_json_error(array(
            'message' => __('Unexpected error', 'eurofaktura')
        ));
        wp_die();
    }

        /**
     * Handles AJAX request for generating PDF document.
     */
    public function parcel_pdf()
    {
        if (!wp_verify_nonce($_POST['_wpnonce'])) {
            wp_send_json_error(array(
                'message' => 'Could not verify nonce'
            ));
            exit;
        }
        $document_id = sanitize_text_field($_POST['document_id']);

        $client = Eurofaktura_http_client::instance();
        $response = $client->post_request('parcel.pdf', ['document_id' => strval($document_id)]);

        if (!empty($response)) {
            $response_status_code = $response->getStatusCode();

            if ($response_status_code === 200) {
                $response_body = $response->getBody()->getContents();
                $content_disposition = $response->getHeader('Content-Disposition');

                $filename = 'parcel.pdf';

                if (!empty($content_disposition)) {
                    preg_match('/filename="([^"]+)"/', $content_disposition[0], $matches);
                    if (count($matches) > 1) {
                        $filename = $matches[1];
                    }
                    wp_send_json_success(array(
                        'filename' => $filename,
                        'contents' => base64_encode($response_body)
                    ));
                    wp_die();
                }
            }
        }
        wp_send_json_error();
        wp_die();
    }

    /**
     * Handles AJAX request for checking and updating order stock.
     */
    public function order_stock()
    {
        if (!wp_verify_nonce($_POST['_wpnonce'])) {
            wp_send_json_error(array(
                'message' => 'Could not verify nonce'
            ));
            exit;
        }
        $document_id = sanitize_text_field($_POST['document_id']);

        $client = Eurofaktura_http_client::instance();
        $response = $client->post_request('order.stock', ['document_id' => strval($document_id)]);

        $response_message = 'Unexpected error';
        if (!empty($response)) {

            $response_status_code = $response->getStatusCode();
            $response_body = $response->getBody()->getContents();
            $response_body = json_decode($response_body, true);

            if (isset($response_body['message'])) {
                $response_message = $response_body['message'];
            }

            if ($response_status_code === 200) {

                $response_body_data = $response_body['data'];
                $table = $response_body_data['table'];
                $modal_contents_html = '<p class="eurofaktura-table-title">' .  $response_body_data['message'] . '</p>';
                $modal_contents_html .= '<section class="eurofaktura-section">';
                $modal_contents_html .= '<div style="overflow-x:auto;">';
                $modal_contents_html .= '<table class="eurofaktura-custom-table">';
                $modal_contents_html .= '<thead>';
                $modal_contents_html .= '<tr>';
                foreach ($table['headers'] as $header) {
                    $modal_contents_html .= '<th style="text-align: left;">' . $header['value'] . '</th>';
                }
                $modal_contents_html .= '</tr>';
                $modal_contents_html .= '</thead>';
                $modal_contents_html .= '<tbody>';
                foreach ($table['rows'] as $row) {
                    $modal_contents_html .= '<tr>';
                    foreach ($row as $value) {
                        $text_align = isset($value['align']) ? $value['align'] : 'right';
                        if (isset($value['color'])) {
                            $modal_contents_html .= '<td style="text-align: ' . $text_align . '; color: ' . $value['color'] . '">' . $value['value'] . '</td>';
                        } else {
                            $modal_contents_html .= '<td style="text-align: ' . $text_align . ';">' . $value['value'] . '</td>';
                        }
                    }
                    $modal_contents_html .= '</tr>';
                }
                $modal_contents_html .= '</tbody>';
                $modal_contents_html .= '</table>';
                $modal_contents_html .= '</div>';
                $modal_contents_html .= '</section>';

                wp_send_json_success(
                    array(
                        'modal_contents_html' => $modal_contents_html,
                        'modal_title' => $response_body_data['title'],
                    )
                );
                wp_die();
            }
            wp_send_json_error(array(
                'message' => $response_message
            ));
            wp_die();
        }
    }
}

// Instantiate the class
return new Eurofaktura_Order_Meta_Box();
