<?php
// If this file is called directly, abort.
if (!defined('ABSPATH')) {
    exit;
}

/**
 * Class Eurofaktura_Orders_Overview_Bulk_Actions
 *
 * This class handles bulk actions for orders in the admin overview.
 *
 * @since      1.1.0
 * @package    eurofaktura
 * @subpackage eurofaktura/includes/admin/orders-overview
 */
class Eurofaktura_Orders_Overview_Bulk_Actions
{
    const CACHE_KEY = 'eurofaktura_bulk_actions_cache_array';

    /**
     * Retrieve the cached bulk actions array.
     *
     * @return array
     */
    private function get_eurofaktura_bulk_actions_cache_array(): array
    {
        $cached_data = get_transient(self::CACHE_KEY);

        if (!is_array($cached_data)) {
            $client = Eurofaktura_Http_Client::instance();
            $response = $client->post_request('order.massActions.list', null);

            if (!empty($response)) {
                $response_status_code = $response->getStatusCode();
                $response_body = json_decode($response->getBody()->getContents(), true);

                if ($response_status_code === 200) {
                    $response_body_data = $response_body['data'];

                    if (!empty($response_body_data) && is_array($response_body_data)) {
                        $this->set_eurofaktura_bulk_actions_cache_array($response_body_data);
                    }
                    $cached_data = get_transient(self::CACHE_KEY);
                }
            }
        }
        return is_array($cached_data) ? $cached_data : [];
    }

    /**
     * Set the bulk actions cache array with a transient.
     *
     * @param array $data_array
     */
    private function set_eurofaktura_bulk_actions_cache_array(array $data_array)
    {
        // Set transient with a 5-minute expiration time
        set_transient(self::CACHE_KEY, $data_array, 5 * MINUTE_IN_SECONDS);
    }

    public function __construct()
    {
        // Remove parameters from request to prevent the redirect action to eurofaktura
        $_SERVER['REQUEST_URI'] = remove_query_arg(
            [
                'eurofaktura_bulk_action',
                'eurofaktura_link',
                'eurofaktura_message',
                'eurofaktura_message_type',
                'eurofaktura_nonce',
            ],
            $_SERVER['REQUEST_URI']
        );

        add_filter('bulk_actions-edit-shop_order', [$this, 'add_bulk_actions'], 20);
        add_action('handle_bulk_actions-edit-shop_order', [$this, 'handle_bulk_actions'], 20, 3);
        add_filter('bulk_actions-woocommerce_page_wc-orders', [$this, 'add_bulk_actions'], 20);
        add_action('handle_bulk_actions-woocommerce_page_wc-orders', [$this, 'handle_bulk_actions'], 20, 3);
        add_action('admin_notices', [$this, 'bulk_action_admin_notice']);
    }

    /**
     * Add bulk actions to the provided array.
     *
     * @param array $actions
     *
     * @return array
     */
    public function add_bulk_actions($actions)
    {
        if (get_option("eurofaktura_connected")) {
            $bulk_actions_array = $this->get_eurofaktura_bulk_actions_cache_array();
            if (!empty($bulk_actions_array) && is_array($bulk_actions_array)) {
                foreach ($bulk_actions_array as $action) {
                    $actions[$action['id']] = $action['title'];
                }
            }
        }

        return $actions;
    }

    /**
     * Handle bulk actions.
     *
     * @param string $redirect
     * @param string $doaction
     * @param array  $object_ids
     *
     * @return string
     */
    public function handle_bulk_actions($redirect, $doaction, $object_ids)
    {
        $actions = $this->get_eurofaktura_bulk_actions_cache_array();

        foreach ($actions as $action) {
            if ($action['id'] == $doaction) {
                $order_array = array_map(
                    function ($id) {
                        return eurofaktura_map_order_for_request(wc_get_order($id));
                    },
                    $object_ids
                );

                $data = [
                    'mass_action' => $doaction,
                    'orders'      => $order_array,
                ];

                $client = Eurofaktura_Http_Client::instance();
                $response = $client->post_request('order.massActions.handle', $data);

                if (!empty($response)) {
                    $response_status_code = $response->getStatusCode();
                    $response_body = json_decode($response->getBody()->getContents(), true);
                    $redirect = add_query_arg(['eurofaktura_nonce' => wp_create_nonce()], $redirect);
                    if ($response_status_code === 200) {
                        $link = $response_body['data'];
                        $redirect = add_query_arg(['eurofaktura_bulk_action' => true, 'eurofaktura_link' => urlencode(urldecode($link))], $redirect);
                    } else {
                        $message = $response_body['message'];
                        $redirect = add_query_arg(['eurofaktura_bulk_action' => true, 'eurofaktura_message' => $message, 'eurofaktura_message_type' => 'error'], $redirect);
                    }
                }
            }
        }

        return $redirect;
    }

    /**
     * Display admin notices for bulk actions.
     */
    public function bulk_action_admin_notice()
    {
        if (isset($_REQUEST['eurofaktura_nonce']) && !empty($_REQUEST['eurofaktura_nonce'])) {
            if (!wp_verify_nonce($_REQUEST['eurofaktura_nonce'])) {
                wp_die('Nonce error');
            }
            if (isset($_REQUEST['eurofaktura_bulk_action']) && !empty($_REQUEST['eurofaktura_bulk_action'])) {
                if (isset($_REQUEST['eurofaktura_link']) && !empty($_REQUEST['eurofaktura_link'])) {
                    printf(
                        '<script>window.open("%s", "eurofaktura");</script>',
                        esc_url_raw($_REQUEST['eurofaktura_link'])
                    );
                }
                if (isset($_REQUEST['eurofaktura_message']) && !empty($_REQUEST['eurofaktura_message'])) {
                    $notice_type = isset($_REQUEST['eurofaktura_message_type']) && ($_REQUEST['eurofaktura_message_type'] === 'error') ? 'error' : '';
                    printf(
                        '<div id="message" class="notice %s is-dismissible"><p>%s</p></div>',
                        esc_attr($notice_type),
                        esc_html($_REQUEST['eurofaktura_message'])
                    );
                }
            }
        }
    }
}

// Instantiate the class
new Eurofaktura_Orders_Overview_Bulk_Actions();
