<?php

// If this file is called directly, abort.
if (!defined('ABSPATH')) exit;
if (defined('Eurofaktura_settings')) exit;

/**
 * Extends the WC_Settings_Page class
 *
 * @since       1.0.0
 *
 * @package     eurofaktura
 *  @subpackage eurofaktura/includes/admin/settings
 *
 */
class Eurofaktura_settings extends WC_Settings_Page
{

    /**
     * Constructor
     * @since  1.0
     */
    public function __construct()
    {
        $this->id = 'eurofaktura';
        $this->label = eurofaktura_get_title_full();

        // Define all hooks instead of inheriting from parent                    
        add_filter('woocommerce_settings_tabs_array', array($this, 'add_settings_page'), 20);
        add_action('woocommerce_sections_' . $this->id, array($this, 'output_sections'));
        add_action('woocommerce_settings_' . $this->id, array($this, 'output'));
        add_action('woocommerce_settings_save_' . $this->id, array($this, 'save'));
    }

    /**
     * Get settings array
     *
     * @return array
     */
    public function get_settings()
    {
        $settings = array();
        include 'eurofaktura-settings-general.php';
        return apply_filters('woocommerce_get_settings_' . $this->id, $settings);
    }

    /**
     * Output the settings
     */
    public function output()
    {
        $settings = $this->get_settings();
        WC_Admin_Settings::output_fields($settings);
    }

    /**
     * Save settings
     *
     * @since 1.0
     */
    public function save()
    {
        $settings = $this->get_settings();
        WC_Admin_Settings::save_fields($settings);

        if ($this->check_mandatory_data() == true) {
            $this->check_connection();
        }
    }

    /**
     * Check if mandatory data is set on setting page
     *
     * @since 1.0
     */
    private function check_mandatory_data(): bool
    {
        $notice_key = 'mandatory_data';
        Eurofaktura_admin_notices::instance()->remove_admin_notice($notice_key);

        $api_credentials_id = get_option('eurofaktura_api_credentials_id');
        $webhook_id = get_option('eurofaktura_webhook_id');
        $missing_data = [];

        if ($api_credentials_id == null || $api_credentials_id == 0) {
            $missing_data[] = __('Api key', 'eurofaktura');
        }

        if ($webhook_id == null || $webhook_id == 0) {
            $missing_data[] = __('Webhook', 'eurofaktura');
        }

        if (!empty($missing_data)) {
            $notice_message = __('Missing mandatory data: ', 'eurofaktura') . implode(', ', $missing_data);
            $admin_notices = Eurofaktura::instance();
            $admin_notices->add_admin_notice($notice_message, $notice_key, 'error');
            return false;
        } else {
            return true;
        }
    }
    private function check_connection()
    {
        update_option('eurofaktura_connected', false);
        delete_transient(Eurofaktura_orders_overview_bulk_actions::CACHE_KEY);

        $client = Eurofaktura_http_client::instance();
        $response = $client->post_request('connection.check', null);

        eurofaktura_add_log_debug(json_encode($response));

        $webhook = get_option('eurofaktura_webhook_id');
        $webhook = eurofaktura_get_active_webhook($webhook);
        // translators: %1$s -> Url of selected webhook. 
        $notice_message = sprintf(__('No response from server on url: %1$s. Please check if selected Webhook Url is correct.', 'eurofaktura'), eurofaktura_get_webhook_delivery_url($webhook));
        $notice_type = 'error';
        if (!empty($response) and !is_null($response)) {

            $response_status_code = $response->getStatusCode();
            $response_body = $response->getBody()->getContents();
            $response_body = json_decode($response_body, true);

            $response_body_custom_status = '';
            if (isset($response_body['status'])) {
                $response_body_custom_status = $response_body['status'];
            }
            if (isset($response_body['message'])) {
                $notice_message = $response_body['message'];
            }
            if ($response_status_code === 200 && $response_body_custom_status === 'connectedSuccess') {
                update_option('eurofaktura_connected', true);
                $notice_type = 'success';

                if (isset($response_body['data']['productName'])) {
                    eurofaktura_set_title($response_body['data']['productName']);
                }
            }
        }
        Eurofaktura_admin_notices::instance()->add_admin_notice($notice_message, 'eurofaktura_settings',  $notice_type);
    }
}

return new Eurofaktura_settings();
