<?php

// If this file is called directly, abort.
if (!defined('ABSPATH')) exit;

/**
 * Manages and displays admin notices.
 *
 * @since      1.0.0
 *
 * @package    eurofaktura
 * @subpackage eurofaktura/includes
 */

class Eurofaktura_admin_notices
{
    private static $instance;

    /**
     * Constructor.
     *
     * Sets up the actions and hooks for handling admin notices.
     */
    private function __construct()
    {
        // Constructor code here
        add_action('admin_notices', array($this, 'show_admin_notices'));
    }

    /**
     * Returns the singleton instance of the class.
     *
     * @return Eurofaktura_admin_notices Singleton instance of Eurofaktura_admin_notices.
     */
    public static function instance()
    {
        if (self::$instance === null) {
            self::$instance = new self();
        }
        return self::$instance;
    }

    /**
     * Retrieves the stored admin notices from the options.
     *
     * @return array Array of admin notices.
     */
    public function get_admin_notices()
    {
        $notices_option = get_option('eurofaktura_admin_notices');
        $notices = $notices_option ? maybe_unserialize($notices_option) : array();
        if (!is_array($notices)) {
            $notices = array();
        }
        return $notices;
    }

    /**
     * Adds a new admin notice.
     *
     * @param string $message      The message to display in the notice.
     * @param string $key          Unique key for the notice.
     * @param string|null $class   CSS class for the notice ('success', 'error', 'warning', 'info').
     * @param bool $permanent      Whether the notice should be permanent.
     * @param bool $dismissible    Whether the notice is dismissible.
     */
    public function add_admin_notice($message, $key, $class = null, $permanent = false, $dismissible = true)
    {
        // Remove already existing notice eventually
        $this->remove_admin_notice($key);

        // Get existing notices
        $notices = $this->get_admin_notices();

        // Add notice to notices
        $notice = array(
            'message' => $message,
            'key' => $key,
            'permanent' => $permanent,
            'dismissible' => $dismissible,
        );
        if ($class) {
            $notice['class'] = $class;
        }
        $notices[] = $notice;

        // Save extended notices array
        update_option('eurofaktura_admin_notices', maybe_serialize($notices));
    }

    /**
     * Removes a specific admin notice by key.
     *
     * @param string $key Key of the notice to be removed.
     */
    public function remove_admin_notice($key)
    {
        $notices_option = get_option('eurofaktura_admin_notices');
        if ($notices_option) {
            // Unset notice by id
            $notices = maybe_unserialize($notices_option);
            foreach ($notices as $i => $notice) {
                if ($notice['key'] === $key) {
                    unset($notices[$i]);
                }
            }

            // Save updated notices array
            update_option('eurofaktura_admin_notices', maybe_serialize($notices));
        }
    }

    /**
     * Removes all admin notices.
     */
    public function remove_admin_notice_all()
    {
        $notices_option = get_option('eurofaktura_admin_notices');
        if ($notices_option) {
            // Unset all notices
            $notices = maybe_unserialize($notices_option);
            foreach ($notices as $i => $notice) {
                unset($notices[$i]);
            }

            // Save updated notices array
            update_option('eurofaktura_admin_notices', maybe_serialize($notices));
        }
    }

    /**
     * Displays admin notices in the WordPress admin.
     */
    public function show_admin_notices()
    {
        if (get_option('eurofaktura_admin_notices')) {
            foreach ($this->get_admin_notices() as $notice) {
                $class = isset($notice['class']) ? $notice['class'] : null;
                $dismissible = isset($notice['dismissible']) ? $notice['dismissible'] : true;
                $dismissible_class = $dismissible ? ' is-dismissible' : '';
                printf('<div class="notice notice-%1$s' . esc_html($dismissible_class) . '"><p>%2$s</p></div>', esc_attr($class), '<strong>' . wp_kses_post($notice['message']) . '</strong>');

                // Remove notice if not permanent
                if (!$notice['permanent']) {
                    $this->remove_admin_notice($notice['key']);
                }
            }
        }
    }
}
