<?php

// If this file is called directly, abort.
if (!defined('ABSPATH')) exit;

use GuzzleHttp\Exception\RequestException;
use GuzzleHttp\Exception\ConnectException;

/**
 * Class Eurofaktura_http_client
 * 
 * Handles HTTP client functionality for WCER plugin.
 *
 * @since      1.0.0
 * 
 * @package    eurofaktura
 * @subpackage eurofaktura/includes
 */

class Eurofaktura_http_client
{
    protected static $_instance = null;
    protected $webhook = null;
    protected $api_credentials = null;
    public $client = null;

    /**
     * Returns the singleton instance of the class.
     *
     * @return Eurofaktura_http_client Singleton instance of Eurofaktura_http_client.
     */
    public static function instance()
    {
        if (is_null(self::$_instance)) {
            self::$_instance = new self();
        }
        return self::$_instance;
    }

    /**
     * Constructor.
     *
     * Initializes the HTTP client with credentials.
     */
    public function __construct()
    {
        $this->set_credentials();
        if ($this->webhook && $this->api_credentials) {
            $this->init_client();
        }
    }

    /**
     * Sets the credentials for the HTTP client.
     */
    public function set_credentials()
    {
        $webhook_id = get_option('eurofaktura_webhook_id');
        $this->webhook = eurofaktura_get_active_webhook($webhook_id);

        $api_credentials_id = get_option('eurofaktura_api_credentials_id');
        $this->api_credentials = eurofaktura_get_api_credentials($api_credentials_id);
    }

    /**
     * Initializes the Guzzle HTTP client.
     */
    private function init_client()
    {
        require plugin_dir_path(dirname(__FILE__)) . '/vendor/autoload.php';

        $base_uri = eurofaktura_get_webhook_delivery_url($this->webhook);

        //This can hapen if user directly disables webhook    
        if (empty($base_uri)) return null;
        $api_key = eurofaktura_get_api_credentials_key($this->api_credentials);

        // Headers for the Guzzle HTTP client
        $headers = [
            'Accept-Encoding' => 'deflate',
            'Content-Type' => 'application/json',
            'Version' => EUROFAKTURA_VERSION,
            'User-Agent' => 'Eurofaktura',
            'X-WP-Plugin-Name' => 'Eurofaktura',
            'X-WP-Language-Name' =>  eurofaktura_get_language_name_for_locale(),
            'X-WC-Api-Truncated-Key' => $api_key,
            'X-WCER-Webhook-Action' => null
        ];

        $this->client = new GuzzleHttp\Client([
            'base_uri' => $base_uri,
            'headers' => $headers
        ]);
    }

    /**
     * Sends a POST request using the initialized client.
     *
     * @param string $action   The action to perform.
     * @param array|null $contents  The contents to include in the request body.
     *
     * @return mixed|null The Guzzle response or null on failure.
     */
    public function post_request(string $action, ?array $contents)
    {
        try {
            $client = $this->client;
            if (empty($client)) return null;
            $data = [];
            $data['headers']['X-Eurofaktura-Webhook-Action'] = $action;

            if (isset($contents)) {
                $data['json'] = $contents;
            }

            $response = $client->request('POST', '', $data);
            return $response;
        } catch (RequestException $ex) {
            if ($ex->hasResponse()) {
                $response = $ex->getResponse();
                return $response;
            } else {
                return null;
            }
        } catch (ConnectException $ex) {
            return null;
        }
    }
}
