<?php
// If this file is called directly, abort.
if (!defined('ABSPATH')) {
    exit;
}

/**
 * Eurofaktura core functions.
 * Extended from WooCommerce/includes/wc-core-function.php
 *
 * @since      1.0.0
 * @package    eurofaktura
 * @subpackage eurofaktura/includes
 */

// Include core functions
require_once plugin_dir_path(dirname(__FILE__)) . 'includes/eurofaktura-webhook-functions.php';
require_once plugin_dir_path(dirname(__FILE__)) . 'includes/eurofaktura-api-functions.php';
require_once plugin_dir_path(dirname(__FILE__)) . 'includes/eurofaktura-loging-functions.php';
require_once plugin_dir_path(dirname(__FILE__)) . 'includes/eurofaktura-order-functions.php';

/**
 * Gets the icon source URL based on the provided symbol.
 *
 * @since 1.0.0
 *
 * @param string $symbol The symbol associated with the icon.
 *
 * @return string The URL of the icon source.
 */
function eurofaktura_get_icon_src_from_symbol($symbol)
{
    // Define the mapping of symbols to their corresponding icon source URLs
    $symbol_mapping = array(
        "fileInvoice"     => "font-awesome-file-invoice.svg",
        "boxes"           => "font-awesome-boxes-stacked.svg",
        "gearSettings"    => "font-awesome-gear.svg",
        "truck"           => "font-awesome-truck-solid.svg",
        "shippingFast"    => "font-awesome-truck-fast-solid.svg",
    );

    // Check if the symbol is in the mapping, otherwise default to "fileInvoice"
    $icon_filename = isset($symbol_mapping[$symbol]) ? $symbol_mapping[$symbol] : "font-awesome-file-invoice.svg";

    // Build and return the complete icon source URL
    return plugins_url("/eurofaktura/assets/icons/" . $icon_filename);
}

/**
 * Gets the language name based on the current locale.
 *
 * @since 1.0.0
 *
 * @return string The name of the language based on the current locale.
 */
function eurofaktura_get_language_name_for_locale()
{
    // Get the current locale
    $locale = get_locale();

    // Extract the language code from the locale
    $language_code = substr($locale, 0, 2);

    // Map language codes to their corresponding language names
    $language_map = array(
        'sl' => 'Slovene',
        'en' => 'English',
        'cs' => 'Czech',
        'hr' => 'Croatian',
        'sr' => 'Serbian',
        'de' => 'German',
        'it' => 'Italian',
    );

    // Default to English if the language code is not found
    $language_name = isset($language_map[$language_code]) ? $language_map[$language_code] : 'English';

    return $language_name;
}

/**
 * Set the title for Eurofaktura.
 *
 * @since 1.1.0
 * 
 * @param string $title The title to set.
 */
function eurofaktura_set_title($title)
{
    $sanitized_title = sanitize_text_field($title);
    update_option('eurofaktura_title', $sanitized_title);
}

/**
 * Get the title for Eurofaktura.
 * 
 * @since 1.1.0
 *
 * @return string The title.
 */
function eurofaktura_get_title()
{
    $title = get_option('eurofaktura_title');
    if(empty($title)) $title = EUROFAKTURA_TITLE;
    return esc_attr($title);
}

/**
 * Get the full title for Eurofaktura eg: (Eurofaktura (e-racuni.com))
 * 
 * @since 1.1.3
 *
 * @return string The title.
 */
function eurofaktura_get_title_full()
{
    $product_title = eurofaktura_get_title();

    if($product_title == EUROFAKTURA_TITLE) return $product_title;
    return esc_attr(sprintf('Eurofaktura (%s)', eurofaktura_get_title()));
}

/**
 * Check if the current user is an administrator.
 * 
 * @since 1.1.3
 *
 * @return bool True if the user is an administrator, false otherwise.
 */
function eurofaktura_is_admin()
{
    return current_user_can('administrator');
}
