<?php

// If this file is called directly, abort.
if (!defined('ABSPATH')) exit;

/**
 * Eurofaktura custom order functions
 *
 * @since      1.0.4
 *
 * @package    eurofaktura
 * @subpackage eurofaktura/includes
 */

/**
 * Maps an order object to an array for API request.
 *
 * @since 1.0.4
 *
 * @param WC_Order $order The WooCommerce order object.
 *
 * @return array An array containing mapped order data for API request.
 */
function eurofaktura_map_order_for_request($order)
{
    $order_array = array(
        'id' => $order->ID,
        'order_number' => $order->get_order_number(),
        'created_at' => $order->get_date_created()->format('Y-m-d\TH:i:s\Z'),
        'updated_at' => $order->get_date_modified()->format('Y-m-d\TH:i:s\Z'),
        'completed_at' => $order->get_date_completed() ? $order->get_date_completed()->format('Y-m-d\TH:i:s\Z') : null,
        'status' => $order->get_status(),
        'currency' => $order->get_currency(),
        'total' => $order->get_total(),
        'subtotal' => $order->get_subtotal(),
        'total_line_items_quantity' => $order->get_item_count(),
        'total_tax' => $order->get_total_tax(),
        'total_shipping' => $order->get_total_shipping(),
        'cart_tax' => $order->get_cart_tax(),
        'shipping_tax' => $order->get_shipping_tax(),
        'total_discount' => $order->get_total_discount(),
        'shipping_methods' => $order->get_shipping_method(),
        'payment_details' => array(
            'method_id' => $order->get_payment_method(),
            'method_title' => $order->get_payment_method_title(),
            'paid' => $order->is_paid(),
        ),
        'billing_address' => array(
            'first_name' => $order->get_billing_first_name(),
            'last_name' => $order->get_billing_last_name(),
            'company' => $order->get_billing_company(),
            'address_1' => $order->get_billing_address_1(),
            'address_2' => $order->get_billing_address_2(),
            'city' => $order->get_billing_city(),
            'state' => $order->get_billing_state(),
            'postcode' => $order->get_billing_postcode(),
            'country' => $order->get_billing_country(),
            'email' => $order->get_billing_email(),
            'phone' => $order->get_billing_phone(),
        ),
        'shipping_address' => array(
            'first_name' => $order->get_shipping_first_name(),
            'last_name' => $order->get_shipping_last_name(),
            'company' => $order->get_shipping_company(),
            'address_1' => $order->get_shipping_address_1(),
            'address_2' => $order->get_shipping_address_2(),
            'city' => $order->get_shipping_city(),
            'state' => $order->get_shipping_state(),
            'postcode' => $order->get_shipping_postcode(),
            'country' => $order->get_shipping_country(),
        ),
        'note' => $order->get_customer_note(),
        'customer_ip' => $order->get_customer_ip_address(),
        'customer_user_agent' => $order->get_customer_user_agent(),
        'customer_id' => $order->get_customer_id(),
        'view_order_url' => $order->get_view_order_url(),
        'customer' => array(
            'id' => $order->get_customer_id(),
            'email' => $order->get_billing_email(),
            'first_name' => $order->get_billing_first_name(),
            'last_name' => $order->get_billing_last_name(),
            'username' => '',
            // You might need to populate this if available
            'last_order_id' => $order->get_id(),
            'last_order_date' => $order->get_date_created()->format('Y-m-d\TH:i:s\Z'),
            'orders_count' => wc_get_customer_order_count($order->get_customer_id()),
            'total_spent' => wc_get_customer_total_spent($order->get_customer_id()),
            'avatar_url' => '',
            // You might need to populate this if available
            'billing_address' => array(
                'first_name' => $order->get_billing_first_name(),
                'last_name' => $order->get_billing_last_name(),
                'company' => $order->get_billing_company(),
                'address_1' => $order->get_billing_address_1(),
                'address_2' => $order->get_billing_address_2(),
                'city' => $order->get_billing_city(),
                'state' => $order->get_billing_state(),
                'postcode' => $order->get_billing_postcode(),
                'country' => $order->get_billing_country(),
                'email' => $order->get_billing_email(),
                'phone' => $order->get_billing_phone(),
            ),
            'shipping_address' => array(
                'first_name' => $order->get_shipping_first_name(),
                'last_name' => $order->get_shipping_last_name(),
                'company' => $order->get_shipping_company(),
                'address_1' => $order->get_shipping_address_1(),
                'address_2' => $order->get_shipping_address_2(),
                'city' => $order->get_shipping_city(),
                'state' => $order->get_shipping_state(),
                'postcode' => $order->get_shipping_postcode(),
                'country' => $order->get_shipping_country(),
            ),
        ),
    );

    $line_items = array();
    foreach ($order->get_items() as $item_id => $item) {
        $product = $item->get_product();
        $line_item = array(
            'id' => $item_id,
            'subtotal' => wc_format_decimal($item->get_subtotal(), 2),
            'subtotal_tax' => wc_format_decimal($item->get_subtotal_tax(), 2),
            'total' => wc_format_decimal($item->get_total(), 2),
            'total_tax' => wc_format_decimal($item->get_total_tax(), 2),
            'price' => wc_format_decimal($product->get_price(), 2),
            'quantity' => $item->get_quantity(),
            'tax_class' => $item->get_tax_class(),
            'name' => $product->get_name(),
            'product_id' => $product->get_id(),
            'sku' => $product->get_sku(),
            'meta' => array(),
            // You can add product meta data here if needed
        );
        $line_items[] = $line_item;
    }
    $order_array['line_items'] = $line_items;

    // 'shipping_lines' section
    $shipping_lines = array();
    foreach ($order->get_items('shipping') as $shipping_item_id => $shipping_item) {
        $shipping_lines[] = array(
            'id' => $shipping_item_id,
            'method_id' => $shipping_item->get_method_id(),
            'method_title' => $shipping_item->get_method_title(),
            'total' => wc_format_decimal($shipping_item->get_total(), 2),
            'total_tax' => wc_format_decimal($shipping_item->get_total_tax(), 2),
            // Add more fields as needed
        );
    }
    $order_array['shipping_lines'] = $shipping_lines;

    // 'tax_lines' section
    $tax_lines = array();
    foreach ($order->get_taxes() as $tax_item) {
        $tax_lines[] = array(
            'id' => $tax_item->get_rate_id(),
            'rate_code' => $tax_item->get_rate_code(),
            'rate_id' => $tax_item->get_rate_id(),
            'label' => $tax_item->get_label(),
            'total' => wc_format_decimal($tax_item->get_tax_total(), 2),
            'compound' => $tax_item->is_compound() ? true : false,
            // Add more fields as needed
        );
    }
    $order_array['tax_lines'] = $tax_lines;

    // 'fee_lines' section
    $fee_lines = array();
    foreach ($order->get_items('fee') as $fee_item_id => $fee_item) {
        $fee_lines[] = array(
            'id' => $fee_item_id,
            'name' => $fee_item->get_name(),
            'total' => wc_format_decimal($fee_item->get_total(), 2),
            // Add more fields as needed
        );
    }
    $order_array['fee_lines'] = $fee_lines;

    // 'coupon_lines' section
    $coupon_lines = array();
    foreach ($order->get_items('coupon') as $coupon_item_id => $coupon_item) {
        $coupon_lines[] = array(
            'id' => $coupon_item_id,
            'code' => $coupon_item->get_code(),
            // Add more fields as needed
        );
    }
    $order_array['coupon_lines'] = $coupon_lines;

    return $order_array;
}
